//============================================================================
//
// SPI to RGB initiaize routine for CFAF480640D0-035FN / HX8363
//
// 2019-12-06 Crystalfontz America, Inc. / Brent A. Crosby
//---------------------------------------------------------------------------
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
//
//
//============================================================================
//This code was used on a Seeeduino (3.3v Arduino clone) to initialize the
//CFAF480640D0-035FN. Once the display is initialized, it requires the use
//of the RGB interface to update the display.
//
//For information on the RGB parameters, please reference the datasheet
//located on the product page.
//https://www.crystalfontz.com/product/cfaf480640d0035fn-480x640-sunlight-readable-tft
//============================================================================
#include <Arduino.h>
#include <SPI.h>

#include "CFA10099_defines.h"
#include "HX8363_Init.h"
//============================================================================
//Register defines for HX8363-A
#define HX8363_A_0x00_NOP 0x00
#define HX8363_A_0x01_SWRESET 0x01
#define HX8363_A_0x06_RDRED 0x06
#define HX8363_A_0x07_RDGREEN 0x07
#define HX8363_A_0x08_RDBLUE 0x08
#define HX8363_A_0x0A_RDDPM 0x0A
#define HX8363_A_0x0B_RDDMADCTL 0x0B
#define HX8363_A_0x0C_RDDCOLMOD 0x0C
#define HX8363_A_0x0D_RDDIM 0x0D
#define HX8363_A_0x0E_RDDSM 0x0E
#define HX8363_A_0x0F_RDDSDR 0x0F
#define HX8363_A_0x10_SLPIN 0x10
#define HX8363_A_0x11_SLPOUT 0x11
#define HX8363_A_0x20_INVOFF 0x20
#define HX8363_A_0x21_INVON 0x21
#define HX8363_A_0x26_GAMSET 0x26
#define HX8363_A_0x28_DISPOFF 0x28
#define HX8363_A_0x29_DISPON 0x29
#define HX8363_A_0x36_MADCTL 0x36
#define HX8363_A_0x3A_COLMOD 0x3A
#define HX8363_A_0xA1_Read_DDB_start 0xA1
#define HX8363_A_0xA8_Read_DDB_continue 0xA8
#define HX8363_A_0xDA_RDID1 0xDA
#define HX8363_A_0xDB_RDID2 0xDB
#define HX8363_A_0xDC_RDID3 0xDC
#define HX8363_A_0x51_WRDISBV 0x51
#define HX8363_A_0x52_RDDISBV 0x52
#define HX8363_A_0x53_WRCTRLD 0x53
#define HX8363_A_0x54_RDCTRLD 0x54
#define HX8363_A_0x55_WRCABC 0x55
#define HX8363_A_0x56_RDCABC 0x56
#define HX8363_A_0x5E_WRCABCMB 0x5E
#define HX8363_A_0x5F_RDCABCMB 0x5F
#define HX8363_A_0xB0_SETINTERNALOSC 0xB0
#define HX8363_A_0xB1_SETPOWER 0xB1
#define HX8363_A_0xB3_SETRGBIF 0xB3
#define HX8363_A_0xB4_SETCYC 0xB4
#define HX8363_A_0xB5_SETBGPVOLTAGE 0xB5
#define HX8363_A_0xB6_SETVCOM 0xB6
#define HX8363_A_0xB8_CPCRC 0xB8
#define HX8363_A_0xB9_SETEXTC 0xB9
#define HX8363_A_0xBA_SETMIPI 0xBA
#define HX8363_A_0xBB_SETOTP 0xBB
#define HX8363_A_0xBC_SETINTDIGVOLTAGE 0xBC
#define HX8363_A_0xBE_LOGICDEBUG 0xBE
#define HX8363_A_0xBF_SETPOWEROPTION 0xBF
#define HX8363_A_0xC0_SETSOURCEOPTION 0xC0
#define HX8363_A_0xC1_SETDGCLUT 0xC1
#define HX8363_A_0xC3_SETID 0xC3
#define HX8363_A_0xC4_SETDDB 0xC4
#define HX8363_A_0xC5_SETROM 0xC5
#define HX8363_A_0xC6_SETECO 0xC6
#define HX8363_A_0xCC_SETPANEL 0xCC
#define HX8363_A_0xFE_SET_SPI_READ_INDEX 0xFE
#define HX8363_A_0xFF_SPIREAD 0xFF
#define HX8363_A_0xE0_SETGAMMA 0xE0
//----------------------------------------------------------------------------
void LCD_Write_Command(uint8_t command)
{
  //Select the LCD
  CLR_DISP_CS;
  uint16_t
      address_and_command;
  //Select the Command register:
  address_and_command = 0x0000;
  //Set the low 8 bits from the caller
  address_and_command |= command;
  // Send the data
  uint16_t
      mask;
  for (mask = 0x0100; 0 != mask; mask >>= 1)
  {
    //Set or clear the data bit = MOSI
    if (0 != (mask & address_and_command))
    {
      SET_MOSI;
    }
    else
    {
      CLR_MOSI;
    }
    //Toggle the clock
    SET_SCK;
    CLR_SCK;
  }
  //De-select the LCD
  SET_DISP_CS;
}
//----------------------------------------------------------------------------
void LCD_Write_Data(uint8_t data)
{
  //Select the LCD
  CLR_DISP_CS;
  uint16_t
      address_and_data;
  //Select the Data register;
  address_and_data = 0x0100;
  //Set the low 8 bits from the caller
  address_and_data |= data;
  // Send the data
  uint16_t
      mask;
  for (mask = 0x0100; 0 != mask; mask >>= 1)
  {
    //Set or clear the data bit = MOSI
    if (0 != (mask & address_and_data))
    {
      SET_MOSI;
    }
    else
    {
      CLR_MOSI;
    }
    //Toggle the clock
    SET_SCK;
    CLR_SCK;
  }
  //De-select the LCD
  SET_DISP_CS;
}
//----------------------------------------------------------------------------
void HX8363_LCD_Init(void)
{
  //Hardware reset the LCD controller
  CLR_DISP_RST;
  delay(20);
  SET_DISP_RST;
  delay(120);

  //Enable extension command
  LCD_Write_Command(HX8363_A_0xB9_SETEXTC);
  LCD_Write_Data(0xFF);
  LCD_Write_Data(0x83);
  LCD_Write_Data(0x63);

  //Set Power
  LCD_Write_Command(HX8363_A_0xB1_SETPOWER);
  LCD_Write_Data(0x81); //1
  LCD_Write_Data(0x30); //2
  LCD_Write_Data(0x07); //3 //0x08 VGH,VGL
  LCD_Write_Data(0x33); //4 //0x33
  LCD_Write_Data(0x02); //5  0x01 2016.08.10
  LCD_Write_Data(0x13); //6
  LCD_Write_Data(0x10); //7
  LCD_Write_Data(0x10); //8
  LCD_Write_Data(0x35); //9
  LCD_Write_Data(0x3D); //10
  LCD_Write_Data(0x1A); //11
  LCD_Write_Data(0x1A); //12
  delay(5);

  // Interface Pixel Format
  LCD_Write_Command(HX8363_A_0x3A_COLMOD);
  LCD_Write_Data(0x60); //60-18bit RGB

  LCD_Write_Command(HX8363_A_0xC0_SETSOURCEOPTION);
  LCD_Write_Data(0x41);
  LCD_Write_Data(0x19);

  LCD_Write_Command(HX8363_A_0xBF_SETPOWEROPTION);
  LCD_Write_Data(0x00);
  LCD_Write_Data(0x10);

  //Set RGB interface related register
  LCD_Write_Command(HX8363_A_0xB3_SETRGBIF);
  LCD_Write_Data(0x01); //00 SET hs,vs,pclk DE polarity

  //Set Display Waveform Cycle
  LCD_Write_Command(HX8363_A_0xB4_SETCYC);
  LCD_Write_Data(0x08); //1 Inversion type setting --2DOT
  LCD_Write_Data(0x12); //2 SON
  LCD_Write_Data(0x72); //3 SOFF
  LCD_Write_Data(0x12); //4 EQS 0x12
  LCD_Write_Data(0x0c); //5 EQN 0x06
  LCD_Write_Data(0x07); //6 0x03 GON
  LCD_Write_Data(0x55); //7 0x54 GOFF
  LCD_Write_Data(0x03); //8 GVSSPQ 0x03
  LCD_Write_Data(0x4D); //9 GVSSP2 0x4E
  LCD_Write_Data(0x00); //10 GVSSP3
  LCD_Write_Data(0x00); //11 GVSS4

  //SET VCOM
  LCD_Write_Command(HX8363_A_0xB6_SETVCOM);
  LCD_Write_Data(0x15);

  //SET PANEL
  LCD_Write_Command(HX8363_A_0xCC_SETPANEL);
  LCD_Write_Data(0x0B); // BGR  GS,SS 03-×óÓÒ·´
  delay(5);

  //SET GAMMMA
  LCD_Write_Command(HX8363_A_0xE0_SETGAMMA);
  LCD_Write_Data(0x01);
  LCD_Write_Data(0x0A);
  LCD_Write_Data(0x4D);
  LCD_Write_Data(0xB0);
  LCD_Write_Data(0x36);
  LCD_Write_Data(0x3F);
  LCD_Write_Data(0x07);
  LCD_Write_Data(0x8D);
  LCD_Write_Data(0x11);
  LCD_Write_Data(0x55);
  LCD_Write_Data(0x18);
  LCD_Write_Data(0xD8);
  LCD_Write_Data(0x96);
  LCD_Write_Data(0xD0);
  LCD_Write_Data(0x13);
  LCD_Write_Data(0x01);
  LCD_Write_Data(0x0A);
  LCD_Write_Data(0x4D);
  LCD_Write_Data(0xB0);
  LCD_Write_Data(0x36);
  LCD_Write_Data(0x3F);
  LCD_Write_Data(0x07);
  LCD_Write_Data(0x8D);
  LCD_Write_Data(0x11);
  LCD_Write_Data(0x55);
  LCD_Write_Data(0x18);
  LCD_Write_Data(0xD5);
  LCD_Write_Data(0x96);
  LCD_Write_Data(0xD0);
  LCD_Write_Data(0x13);
  delay(5);

  LCD_Write_Command(HX8363_A_0xB0_SETINTERNALOSC);
  LCD_Write_Data(0x01);
  LCD_Write_Data(0x47);
  LCD_Write_Data(0x66);

  //Sleep Out (wake)
  LCD_Write_Command(HX8363_A_0x11_SLPOUT);
  delay(120);
  //Display On
  LCD_Write_Command(HX8363_A_0x29_DISPON);
}
//============================================================================
